#!/usr/bin/python
# -*- coding:utf-8 -*- 
import sys, os, string, re
#, urllib
from dommodel import *

class NetowrkExceptionByCurl ( Exception ):
    """Exception subclass for network loading"""
    def __init__( self, message = "" ):
        Exception.__init__( self, message )
        pass
    pass

class JMAEarthQuake:
    """A class of earthquake information from JMA.
    This class not only downloads HTML contents in JMA server but also interpretes the content and acquire main notations."""
    def __init__( self, url = None ):
        self.__flag_local = ( url == None )
        self.__url = url
        self.__address_next  = None
        self.__address_prev  = None
        self.__code_next     = None
        self.__code_prev     = None

        self.__information   = ""
        self.__points        = []

        self.__date_report     = ""
        self.__date_earthquake = ""
        self.__location        = ( "", "", "", "" )
        self.__magnitude       = 0
        self.__tsunami         = None
        self.__read_data( self.__url )
        
        pass

    def get_address_previous( self ):
        """URL of the previously occuurring earthquake."""
        return self.__address_prev

    def get_address_next( self ):
        """URL of the next earthquake"""
        return self.__address_next

    def __read_jump_address( self, elem ):
        if elem == None: return None
        command_jump    = elem.get_attribute( "onclick" )
        if command_jump == None: return None
        m = re.search( "urlJump\\( \"([^\"]+)", command_jump )
        if m: return m.group( 1 )
        return None

    def get_url( self ):
        """The URL of the information."""
        return self.__url

    def __get_base_url():
        return "http://www.jma.go.jp/JMA_HP/jp/quake/"

    get_base_URL = staticmethod( __get_base_url)

    def __read_data( self, url ):
        p = DOMConverter()
        filename_temp = None
        if url == None:
            stream = open( "../test.html" )
        else:
            #url = JMAEarthQuake.get_URL() + "quake.050516072502.03.0.html"
            #stream = urllib.urlopen( url )
            import tempfile, os.path
            filename_temp = tempfile.mktemp( "eqk" )
            os.system( '/usr/bin/curl "%s" -s -o %s' % ( url, filename_temp ) )
            if os.path.exists( filename_temp ) == False:
                raise NetowrkExceptionByCurl( "cannot load the address " + url )
            stream = open( filename_temp )
            pass
        while True:
            line = stream.readline()
            if line == "": break
            p.feed( unicode( line, 'sjis' ).encode( 'utf-8' ) )
            pass

        stream.close()
        if filename_temp != None:
            os.unlink( filename_temp )
            pass
        doc = p.get_document()

        try:
            elem_data = doc.get_element_at_path( "/html/body/table/tr/td" ).next_sibling()
            elem_jump = elem_data.get_element_at_path( "form/table/tr" ).next_sibling().get_element_at_path( "td/input" )

        except: # no data
            raise

        self.__code_prev = address_last = self.__read_jump_address( elem_jump )
        self.__code_next = self.__read_jump_address( elem_jump.next_sibling() )
        url_base = JMAEarthQuake.get_base_URL()
        self.__address_prev = "%squake.%s.html" % ( url_base, self.__code_prev )
        if self.__code_next:
            self.__address_next = "%squake.%s.html" % ( url_base, self.__code_next )
        else:
            self.__address_next = ""
            pass
        elem_info = elem_data.get_element_at_path( "form/table/tr" ).next_sibling().next_sibling().next_sibling().get_element_at_path( "td/table/tr/td" )
        info_text = ""

        children = elem_info.get_children()
        for i in range( 1, len( children ) ):
            child = children[ i ]
            if isinstance( child, Text ): info_text += child.get_text() + "\n"
            pass
        self.__information = info_text
        elem_area_info = elem_info.get_element_at_path( "table/tr" )
        while elem_area_info != None:
            elem     = elem_area_info.get_element_at_path( "td" )
            try:
                area     = elem.get_text().strip()
                elem = elem.next_sibling()
                strength = elem.get_text().strip()
                if self.__resolve_magnitude( strength ) == 0:
                    self.__points.append( ( "", area, strength ) )
                else:
                    elem = elem.next_sibling()
                    point    = elem.get_text().strip()
                    self.__points.append( ( area, strength, point ) )
                    pass
                pass
            except Exception, e:
                pass
            elem_area_info = elem_area_info.next_sibling()
            pass
        pass

    def __repr__( self ):
        return self.__get_string_with_threshold( 0 )

    def __resolve_magnitude( self, magstr ):
        try:
            m = re.search( "(\\d+)", magstr )
            if m: return string.atoi( m.group( 1 ) )
            pass
        except:
            pass
        if magstr.count( "８" ) > 0 : return 8
        if magstr.count( "７" ) > 0 : return 7
        if magstr.count( "６" ) > 0 : return 6
        if magstr.count( "５" ) > 0 : return 5
        if magstr.count( "４" ) > 0 : return 4
        if magstr.count( "３" ) > 0 : return 3
        if magstr.count( "２" ) > 0 : return 2
        if magstr.count( "１" ) > 0 : return 1
        return 0

    def __get_string_with_threshold( self, threshold = 0 ):
        if self.__url != None:
            line = "URL:%s\n" % self.__url
        else:
            line = "URL:\n"
            pass
        line += "PRV:%s\n" % self.__address_prev
        line += "PST:%s\n" % self.__address_next
        line += self.__information
        line += "//-\n"
        flag_cut = False
        for point in self.__points:
            if flag_cut:
                if point[ 0 ] != "":
                    flag_cut = True
                    pass
                else:
                    continue
                pass
            if self.__resolve_magnitude( point[ 1 ] ) <= threshold:
                flag_cut = True
                continue
            line += "%s,%s,%s\n" % ( point[ 0 ], point[ 1 ], point[ 2 ] )
            pass
        return line
        pass

    def get_string( self, size = 4000 ):
        """A converting method from an object to a string with restricted buffer size. This restriction is introduced because of Dashboard widget input of JavaScript.
        If the string is larger than the buffer size, regional information is reduced using neglecting regions of smaller magnitudes."""
        try:
            output = self.__get_string_with_threshold()
            threshold = 1
            while len( output ) > size:
                threshold += 1
                output = self.__get_string_with_threshold( threshold )
                if threshold >= 6: output = output[ 0 : size ]
                pass
            return output
        except:
            return ""
        pass
    pass

if __name__ == "__main__":
    url = "http://www.jma.go.jp/JMA_HP/jp/quake/index.html"
    #url = "http://www.jma.go.jp/JMA_HP/jp/quake/quake.050515160424.03.0.html"

    for i in range( 10 ):
        e = JMAEarthQuake( url )
        output = e.get_string( 1000 )
        print output, len( output )
        url = e.get_address_previous()
        pass
    pass

